/**
 * Visual Blocks Language
 *
 * Copyright 2012 Google Inc.
 * http://code.google.com/p/blockly/
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @fileoverview Helper functions for generating Arduino for blocks.
 * @author gasolin@gmail.com (Fred Lin)
 */
'use strict';

goog.provide('Blockly.Arduino');

goog.require('Blockly.Generator');

/*
 * Arduino Board profiles
 *
 */

var profile = {
    none: {
        description: "",
        cpu: "",
        speed: "",
        digital: [],
        PWM: [],
        dropdownPWM: [],
        analog: [],
        dropdownAnalog: [],
        /*irqonchange: [],*/
        I2C: [],
        SPI: [],
        interrupt: [],
        picture: "media/boards/none.jpg",
        miniPicture: "media/boards/none_mini.jpg",
        miniPicture_hor: "media/boards/none_mini_hor.jpg",
        serial: [],
        serialPin: [],
        upload_arg: "none",
        help_link: ""
    },
    arduino_leonardo: {
        description: "Arduino Leonardo",
        cpu: "atmega32u4",
        speed: "57600",
        digital: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13"],
        dropdownDigital: [["0", "0"], ["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"]],
        PWM: ["3", "5", "6", "9", "10", "11", "13"],
        dropdownPWM: [["3", "3"], ["5", "5"], ["6", "6"], ["9", "9"], ["10", "10"], ["11", "11"], ["13", "13"]],
        analog: ["A0", "A1", "A2", "A3", "A4", "A5", "A6", "A7", "A6", "A7", "A8", "A9", "A10", "A11"],
        dropdownAnalog: [["A0", "A0"], ["A1", "A1"], ["A2", "A2"], ["A3", "A3"], ["A4", "A4"], ["A5", "A5"], ["A6(D4)", "4"], ["A7(D6)", "6"], ["A8(D8)", "8"], ["A9(D9)", "9"], ["A10(D10)", "10"], ["A11(D12)", "12"]],
        /*irqonchange: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"],["A0", "14"], ["A1", "15"], ["A2", "16"], ["A3", "17"], ["A4", "18"], ["A5", "19"]],*/
        I2C: ["2", "3"],
        SPI: ["connect"],
        interrupt: ["3", "2", "0", "1", "7"],
        picture: "media/boards/arduino_leonardo.jpg",
        miniPicture: "media/boards/arduino_leonardo_mini.jpg",
        miniPicture_hor: "media/boards/arduino_leonardo_mini_hor.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["0 (Rx) ; 1 (Tx)", "0"]],
        upload_arg: "arduino:avr:leonardo",
        help_link: "https://www.arduino.cc/en/Main/Arduino_BoardLeonardo"
    },
    arduino_mega: {
        description: "Arduino Mega",
        cpu: "atmega2560",
        speed: "115200",
        digital: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15", "16", "17", "18", "19", "20", "21", "22", "23", "24", "25", "26", "27", "28", "29", "30", "31", "32", "33", "34", "35", "36", "37", "38", "39", "40", "41", "42", "43", "44", "45", "46", "47", "48", "49", "50", "51", "52"],
        dropdownDigital: "attention",
        PWM: ["2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "44", "45", "46"],
        dropdownPWM: [["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"], ["44", "44"], ["45", "45"], ["46", "46"]],
        analog: ["A0", "A1", "A2", "A3", "A4", "A5", "A6", "A7", "A8", "A9", "A10", "A11", "A12", "A13", "A14", "A15"],
        dropdownAnalog: [["A0", "A0"], ["A1", "A1"], ["A2", "A2"], ["A3", "A3"], ["A4", "A4"], ["A5", "A5"], ["A6", "A6"], ["A7", "A7"], ["A8", "A8"], ["A9", "A9"], ["A10", "A10"], ["A11", "A11"], ["A12", "A12"], ["A13", "A13"], ["A14", "A14"], ["A15", "A15"]],
        /*irqonchange : [["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"], ["14", "14"], ["15", "15"], ["50", "50"], ["51", "51"], ["52", "52"], ["53", "53"], ["A8", "62"], ["A9", "63"], ["A10", "64"], ["A11", "65"], ["A12", "66"], ["A13", "67"], ["A14", "68"], ["A15", "69"]],*/
        I2C: ["20", "21"],
        SPI: [["50 (SS)", "50"], ["51 (MOSI)", "51"], ["52 (MISO)", "52"], ["53 (SCK)", "53"]],
        interrupt: ["2", "3", "21", "20", "19", "18"],
        picture: "media/boards/arduino_mega.jpg",
        miniPicture: "media/boards/arduino_mega_mini.jpg",
        miniPicture_hor: "media/boards/arduino_mega_mini_hor.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["0 (Rx) ; 1 (Tx)", "0"], ["19 (Rx1) ; 18 (Tx1)", "19"], ["17 (Rx2) ; 16 (Tx2)", "17"], ["15 (Rx3) ; 14 (Tx3)", "15"]],
        upload_arg: "arduino:avr:mega:cpu=atmega2560",
        help_link: "https://store.arduino.cc/mega-2560-r3"
    },
    arduino_micro: {
        description: "Arduino Micro",
        cpu: "atmega32u4",
        speed: "57600",
        digital: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13"],
        dropdownDigital: [["0", "0"], ["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"]],
        PWM: ["3", "5", "6", "9", "10", "11", "13"],
        dropdownPWM: [["3", "3"], ["5", "5"], ["6", "6"], ["9", "9"], ["10", "10"], ["11", "11"], ["13", "13"]],
        analog: ["A0", "A1", "A2", "A3", "A4", "A5", "A6", "A7", "A6", "A7", "A8", "A9", "A10", "A11"],
        dropdownAnalog: [["A0", "A0"], ["A1", "A1"], ["A2", "A2"], ["A3", "A3"], ["A4", "A4"], ["A5", "A5"], ["A6(D4)", "4"], ["A7(D6)", "6"], ["A8(D8)", "8"], ["A9(D9)", "9"], ["A10(D10)", "10"], ["A11(D12)", "12"]],
        /*irqonchange: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"],["A0", "14"], ["A1", "15"], ["A2", "16"], ["A3", "17"], ["A4", "18"], ["A5", "19"]],*/
        I2C: ["2", "3"],
        SPI: ["connect"],
        interrupt: [["0(Rx)", "0"], ["1(Tx)", "1"], ["2", "2"], ["3", "3"], ["7", "7"]],
        picture: "media/boards/arduino_micro.jpg",
        miniPicture: "media/boards/arduino_micro_mini.jpg",
        miniPicture_hor: "media/boards/arduino_micro_mini_hor.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["0 (Rx) ; 1 (Tx)", "0"]],
        upload_arg: "arduino:avr:micro",
        help_link: "https://store.arduino.cc/arduino-micro"
    },
    arduino_mini: {
        description: "Arduino Mini",
        cpu: "atmega328p",
        speed: "115200",
        digital: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13"],
        dropdownDigital: [["0", "0"], ["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"]],
        PWM: ["3", "5", "6", "9", "10", "11"],
        dropdownPWM: [["3", "3"], ["5", "5"], ["6", "6"], ["9", "9"], ["10", "10"], ["11", "11"]],
        analog: ["A0", "A1", "A2", "A3", "A4", "A5", "A6", "A7"],
        dropdownAnalog: [["A0", "A0"], ["A1", "A1"], ["A2", "A2"], ["A3", "A3"], ["A4", "A4"], ["A5", "A5"], ["A6", "A6"], ["A7", "A7"]],
        I2C: ["A4", "A5"],
        SPI: [["10 (SS)", "10"], ["11 (MOSI)", "11"], ["12 (MISO)", "12"], ["13 (SCK)", "13"]],
        interrupt: [["2", "2"], ["3", "3"]],
        picture: "media/boards/arduino_mini.jpg",
        miniPicture: "media/boards/arduino_mini_mini.jpg",
        miniPicture_hor: "media/boards/arduino_mini_mini_hor.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["0 (Rx) ; 1 (Tx)", "0"]],
        upload_arg: "arduino:avr:mini",
        help_link: "https://store.arduino.cc/arduino-mini-05"
    },
    arduino_nano: {
        description: "Arduino Nano",
        cpu: "atmega328p",
        speed: "115200",
        digital: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13"],
        dropdownDigital: [["0", "0"], ["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"]],
        PWM: ["3", "5", "6", "9", "10", "11"],
        dropdownPWM: [["3", "3"], ["5", "5"], ["6", "6"], ["9", "9"], ["10", "10"], ["11", "11"]],
        analog: ["A0", "A1", "A2", "A3", "A4", "A5", "A6", "A7"],
        dropdownAnalog: [["A0", "A0"], ["A1", "A1"], ["A2", "A2"], ["A3", "A3"], ["A4", "A4"], ["A5", "A5"], ["A6", "A6"], ["A7", "A7"]],
        /*irqonchange: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"],["A0", "14"], ["A1", "15"], ["A2", "16"], ["A3", "17"], ["A4", "18"], ["A5", "19"]],*/
        I2C: ["A4", "A5"],
        SPI: [["10 (SS)", "10"], ["11 (MOSI)", "11"], ["12 (MISO)", "12"], ["13 (SCK)", "13"]],
        interrupt: ["2", "3"],
        picture: "media/boards/arduino_nano.jpg",
        miniPicture: "media/boards/arduino_nano_mini.jpg",
        miniPicture_hor: "media/boards/arduino_nano_mini_hor.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["0 (Rx) ; 1 (Tx)", "0"]],
        upload_arg: "arduino:avr:nano:cpu=atmega328",
        help_link: "https://www.arduino.cc/en/Main/ArduinoBoardNano"
    },
    arduino_pro8: {
        description: "Arduino Pro Mini 3.3V",
        cpu: "atmega328p",
        speed: "57600",
        digital: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13"],
        dropdownDigital: [["0", "0"], ["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"]],
        PWM: ["3", "5", "6", "9", "10", "11"],
        dropdownPWM: [["3", "3"], ["5", "5"], ["6", "6"], ["9", "9"], ["10", "10"], ["11", "11"]],
        analog: ["A0", "A1", "A2", "A3", "A4", "A5"],
        dropdownAnalog: [["A0", "A0"], ["A1", "A1"], ["A2", "A2"], ["A3", "A3"], ["A4", "A4"], ["A5", "A5"]],
        I2C: ["A4", "A5"],
        SPI: [["10 (SS)", "10"], ["11 (MOSI)", "11"], ["12 (MISO)", "12"], ["13 (SCK)", "13"]],
        interrupt: [["2", "2"], ["3", "3"]],
        picture: "media/boards/arduino_pro.jpg",
        miniPicture: "media/boards/arduino_pro_mini.jpg",
        miniPicture_hor: "media/boards/arduino_pro_mini_hor.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["0 (Rx) ; 1 (Tx)", "0"]],
        upload_arg: "arduino:avr:pro:cpu=8MHzatmega328",
        help_link: "https://www.arduino.cc/en/Main/ArduinoBoardProMini"
    },
    arduino_pro16: {
        description: "Arduino Pro Mini 5V",
        cpu: "atmega328p",
        speed: "57600",
        digital: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13"],
        dropdownDigital: [["0", "0"], ["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"]],
        PWM: ["3", "5", "6", "9", "10", "11"],
        dropdownPWM: [["3", "3"], ["5", "5"], ["6", "6"], ["9", "9"], ["10", "10"], ["11", "11"]],
        analog: ["A0", "A1", "A2", "A3", "A4", "A5"],
        dropdownAnalog: [["A0", "A0"], ["A1", "A1"], ["A2", "A2"], ["A3", "A3"], ["A4", "A4"], ["A5", "A5"]],
        I2C: ["A4", "A5"],
        SPI: [["10 (SS)", "10"], ["11 (MOSI)", "11"], ["12 (MISO)", "12"], ["13 (SCK)", "13"]],
        interrupt: [["2", "2"], ["3", "3"]],
        picture: "media/boards/arduino_pro.jpg",
        miniPicture: "media/boards/arduino_pro_mini.jpg",
        miniPicture_hor: "media/boards/arduino_pro_mini_hor.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["0 (Rx) ; 1 (Tx)", "0"]],
        upload_arg: "arduino:avr:pro:cpu=16MHzatmega328",
        help_link: "https://www.arduino.cc/en/Main/ArduinoBoardProMini"
    },
    arduino_uno: {
        description: "Arduino Uno",
        cpu: "atmega328p",
        speed: "115200",
        digital: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13"],
        dropdownDigital: [["0", "0"], ["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"]],
        PWM: ["3", "5", "6", "9", "10", "11"],
        dropdownPWM: [["3", "3"], ["5", "5"], ["6", "6"], ["9", "9"], ["10", "10"], ["11", "11"]],
        analog: ["A0", "A1", "A2", "A3", "A4", "A5"],
        dropdownAnalog: [["A0", "A0"], ["A1", "A1"], ["A2", "A2"], ["A3", "A3"], ["A4", "A4"], ["A5", "A5"]],
        /*irqonchange: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"],["A0", "14"], ["A1", "15"], ["A2", "16"], ["A3", "17"], ["A4", "18"], ["A5", "19"]],*/
        I2C: ["A4", "A5"],
        SPI: [["10 (SS)", "10"], ["11 (MOSI)", "11"], ["12 (MISO)", "12"], ["13 (SCK)", "13"]],
        interrupt: ["2", "3"],
        picture: "media/boards/arduino_uno.jpg",
        miniPicture: "media/boards/arduino_uno_mini.jpg",
        miniPicture_hor: "media/boards/arduino_uno_mini_hor.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["0 (Rx) ; 1 (Tx)", "0"]],
        upload_arg: "arduino:avr:uno",
        help_link: "https://www.arduino.cc/en/Main/ArduinoBoardUno"
    },
    arduino_yun: {
        description: "Arduino Yùn",
        cpu: "atmega32u4",
        speed: "57600",
        digital: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13"],
        dropdownDigital: [["0", "0"], ["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"]],
        PWM: ["3", "5", "6", "9", "10", "11", "13"],
        dropdownPWM: [["3", "3"], ["5", "5"], ["6", "6"], ["9", "9"], ["10", "10"], ["11", "11"], ["13", "13"]],
        analog: ["A0", "A1", "A2", "A3", "A4", "A5", "A6", "A7", "A6", "A7", "A8", "A9", "A10", "A11"],
        dropdownAnalog: [["A0", "A0"], ["A1", "A1"], ["A2", "A2"], ["A3", "A3"], ["A4", "A4"], ["A5", "A5"], ["A6(D4)", "4"], ["A7(D6)", "6"], ["A8(D8)", "8"], ["A9(D9)", "9"], ["A10(D10)", "10"], ["A11(D12)", "12"]],
        /*irqonchange: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"],["A0", "14"], ["A1", "15"], ["A2", "16"], ["A3", "17"], ["A4", "18"], ["A5", "19"]],*/
        I2C: ["A4", "A5"],
        SPI: ["connect"],
        interrupt: ["3", "2", "0", "1", "7"],
        picture: "media/boards/arduino_yun.jpg",
        miniPicture: "media/boards/arduino_yun_mini.jpg",
        miniPicture_hor: "media/boards/arduino_yun_mini_hor.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["0 (Rx) ; 1 (Tx)", "0"]],
        upload_arg: "arduino:avr:yun",
        help_link: "https://www.arduino.cc/en/Main/ArduinoBoardYun"
    },
    lilypad: {
        description: "LilyPad Arduino",
        cpu: "atmega328p",
        speed: "57600",
        digital: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13"],
        dropdownDigital: [["0", "0"], ["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"]],
        PWM: ["3", "5", "6", "9", "10", "11"],
        dropdownPWM: [["3", "3"], ["5", "5"], ["6", "6"], ["9", "9"], ["10", "10"], ["11", "11"]],
        analog: ["A0", "A1", "A2", "A3", "A4", "A5"],
        dropdownAnalog: [["A0", "A0"], ["A1", "A1"], ["A2", "A2"], ["A3", "A3"], ["A4", "A4"], ["A5", "A5"]],
        /*irqonchange: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"],["A0", "14"], ["A1", "15"], ["A2", "16"], ["A3", "17"], ["A4", "18"], ["A5", "19"]],*/
        I2C: ["A4", "A5"],
        SPI: [["10 (SS)", "10"], ["11 (MOSI)", "11"], ["12 (MISO)", "12"], ["13 (SCK)", "13"]],
        interrupt: ["2", "3"],
        picture: "media/boards/lilypad.jpg",
        miniPicture: "media/boards/lilypad_mini.jpg",
        miniPicture_hor: "media/boards/lilypad_mini_hor.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["0 (Rx) ; 1 (Tx)", "0"]],
        upload_arg: "arduino:avr:lilypad",
        help_link: "https://www.arduino.cc/en/Main/ArduinoBoardLilyPad/"
    },
    dagu_rs027: {
        description: "Dagu RS027",
        cpu: "atmega8",
        speed: "19200",
        digital: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13"],
        dropdownDigital: [["0", "0"], ["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"]],
        PWM: ["9", "10", "11"],
        dropdownPWM: [["9", "9"], ["10", "10"], ["11", "11"]],
        analog: ["A0", "A1", "A2", "A3", "A4", "A5"],
        dropdownAnalog: [["A0", "A0"], ["A1", "A1"], ["A2", "A2"], ["A3", "A3"], ["A4", "A4"], ["A5", "A5"]],
        I2C: ["A4", "A5"],
        SPI: [["10 (SS)", "10"], ["11 (MOSI)", "11"], ["12 (MISO)", "12"], ["13 (SCK)", "13"]],
        interrupt: [["2", "2"], ["3", "3"]],
        picture: "media/boards/dagu_rs027.jpg",
        miniPicture: "media/boards/dagu_rs027_mini.jpg",
        miniPicture_hor: "media/boards/dagu_rs027_mini_hor.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["0 (Rx) ; 1 (Tx)", "0"]],
        upload_arg: "arduino:avr:pro:cpu=8MHzatmega168",
        help_link: "https://www.gotronic.fr/art-carte-de-controle-rs027-18765.htm"
    },
    dagu_rs040: {
        description: "Dagu RS040",
        cpu: "atmega328p",
        speed: "57600",
        digital: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13"],
        dropdownDigital: [["0", "0"], ["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"]],
        PWM: ["3", "5", "6", "9", "10", "11"],
        dropdownPWM: [["3", "3"], ["5", "5"], ["6", "6"], ["9", "9"], ["10", "10"], ["11", "11"]],
        analog: ["A0", "A1", "A2", "A3", "A4", "A5"],
        dropdownAnalog: [["A0", "A0"], ["A1", "A1"], ["A2", "A2"], ["A3", "A3"], ["A4", "A4"], ["A5", "A5"]],
        I2C: ["A4", "A5"],
        SPI: [["10 (SS)", "10"], ["11 (MOSI)", "11"], ["12 (MISO)", "12"], ["13 (SCK)", "13"]],
        interrupt: [["2", "2"], ["3", "3"]],
        picture: "media/boards/dagu_rs040.jpg",
        miniPicture: "media/boards/dagu_rs040_mini.jpg",
        miniPicture_hor: "media/boards/dagu_rs040_mini_hor.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["0 (Rx) ; 1 (Tx)", "0"]],
        upload_arg: "arduino:avr:nano:cpu=atmega328",
        help_link: "https://www.gotronic.fr/art-carte-mini-driver-mkii-24795.htm"
    },
    makeblock_mcore: {
        description: "Makeblock mCore for mBot",
        cpu: "atmega328p",
        speed: "115200",
        digital: ["9", "10", "11", "12"],
        dropdownDigital: [["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"]],
        PWM: ["9", "10", "11"],
        dropdownPWM: [["9", "9"], ["10", "10"], ["11", "11"]],
        analog: ["A0", "A1", "A2", "A3"],
        dropdownAnalog: [["A0", "A0"], ["A1", "A1"], ["A2", "A2"], ["A3", "A3"]],
        /*irqonchange: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"],["A0", "14"], ["A1", "15"], ["A2", "16"], ["A3", "17"], ["A4", "18"], ["A5", "19"]],*/
        I2C: ["A4", "A5"],
        SPI: [["10 (SS)", "10"], ["11 (MOSI)", "11"], ["12 (MISO)", "12"], ["13 (SCK)", "13"]],
        interrupt: ["2", "3"],
        picture: "media/boards/makeblock_mcore.jpg",
        miniPicture: "media/boards/makeblock_mcore_mini.jpg",
        miniPicture_hor: "media/boards/makeblock_mcore_mini_hor.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["0 (Rx) ; 1 (Tx)", "0"]],
        upload_arg: "arduino:avr:uno",
        help_link: "https://www.makeblock.com/project/mcore"
    },
    makeblock_megaPi: {
        description: "Makeblock MegaPi",
        cpu: "atmega2560",
        speed: "115200",
        digital: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15", "16", "17", "18", "19", "20", "21", "22", "23", "24", "25", "26", "27", "28", "29", "30", "31", "32", "33", "34", "35", "36", "37", "38", "39", "40", "41", "42", "43", "44", "45", "46", "47", "48", "49", "50", "51", "52"],
        dropdownDigital: "attention",
        PWM: ["2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "44", "45", "46"],
        dropdownPWM: [["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"], ["44", "44"], ["45", "45"], ["46", "46"]],
        analog: ["A0", "A1", "A2", "A3", "A4", "A5", "A6", "A7", "A8", "A9", "A10", "A11", "A12", "A13", "A14", "A15"],
        dropdownAnalog: [["A0", "A0"], ["A1", "A1"], ["A2", "A2"], ["A3", "A3"], ["A4", "A4"], ["A5", "A5"], ["A6", "A6"], ["A7", "A7"], ["A8", "A8"], ["A9", "A9"], ["A10", "A10"], ["A11", "A11"], ["A12", "A12"], ["A13", "A13"], ["A14", "A14"], ["A15", "A15"]],
        /*irqonchange : [["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"], ["14", "14"], ["15", "15"], ["50", "50"], ["51", "51"], ["52", "52"], ["53", "53"], ["A8", "62"], ["A9", "63"], ["A10", "64"], ["A11", "65"], ["A12", "66"], ["A13", "67"], ["A14", "68"], ["A15", "69"]],*/
        I2C: ["20", "21"],
        SPI: [["50 (SS)", "50"], ["51 (MOSI)", "51"], ["52 (MISO)", "52"], ["53 (SCK)", "53"]],
        interrupt: ["2", "3", "21", "20", "19", "18"],
        picture: "media/boards/makeblock_megaPi.jpg",
        miniPicture: "media/boards/makeblock_megaPi_mini.jpg",
        miniPicture_hor: "media/boards/makeblock_megaPi_mini_hor.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["0 (Rx) ; 1 (Tx)", "0"], ["19 (Rx1) ; 18 (Tx1)", "19"], ["17 (Rx2) ; 16 (Tx2)", "17"], ["15 (Rx3) ; 14 (Tx3)", "15"]],
        upload_arg: "arduino:avr:mega:cpu=atmega2560",
        help_link: "https://www.makeblock.com/project/megapi"
    },
    makeblock_orion: {
        description: "Makeblock Me Orion",
        cpu: "atmega328p",
        speed: "115200",
        digital: ["0", "1", "2", "3", "8", "9", "10", "11", "12", "13"],
        dropdownDigital: [["0", "0"], ["1", "1"], ["2", "2"], ["3", "3"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"]],
        PWM: ["3", "10", "11"],
        dropdownPWM: [["3", "3"], ["9", "9"], ["10", "10"], ["11", "11"]],
        analog: ["A0", "A1", "A2", "A3", "A4", "A5", "A6", "A7"],
        dropdownAnalog: [["A0", "A0"], ["A1", "A1"], ["A2", "A2"], ["A3", "A3"], ["A4", "A4"], ["A5", "A5"], ["A6", "A6"], ["A7", "A7"]],
        /*irqonchange: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"],["A0", "14"], ["A1", "15"], ["A2", "16"], ["A3", "17"], ["A4", "18"], ["A5", "19"]],*/
        I2C: ["A4", "A5"],
        SPI: [["10 (SS)", "10"], ["11 (MOSI)", "11"], ["12 (MISO)", "12"], ["13 (SCK)", "13"]],
        interrupt: ["2", "3"],
        picture: "media/boards/makeblock_orion.jpg",
        miniPicture: "media/boards/makeblock_orion_mini.jpg",
        miniPicture_hor: "media/boards/makeblock_orion_mini_hor.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["0 (Rx) ; 1 (Tx)", "0"]],
        upload_arg: "arduino:avr:uno",
        help_link: "https://www.makeblock.com/project/makeblock-orion"
    },
    dfrobot_romeo: {
        description: "RoMeo v2",
        cpu: "atmega32u4",
        speed: "57600",
        digital: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13"],
        dropdownDigital: [["0", "0"], ["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"]],
        PWM: ["3", "5", "6", "9", "10", "11", "13"],
        dropdownPWM: [["3", "3"], ["5", "5"], ["6", "6"], ["9", "9"], ["10", "10"], ["11", "11"], ["13", "13"]],
        analog: ["A0", "A1", "A2", "A3", "A4", "A5", "A6", "A7", "A6", "A7", "A8", "A9", "A10", "A11"],
        dropdownAnalog: [["A0", "A0"], ["A1", "A1"], ["A2", "A2"], ["A3", "A3"], ["A4", "A4"], ["A5", "A5"], ["A6(D4)", "4"], ["A7(D6)", "6"], ["A8(D8)", "8"], ["A9(D9)", "9"], ["A10(D10)", "10"], ["A11(D12)", "12"]],
        I2C: ["connect"],
        SPI: [["16 (MOSI)", "16"], ["14 (MISO)", "14"], ["15 (SCK)", "15"]],
        interrupt: ["3", "2", "0", "1", "7"],
        picture: "media/boards/dfrobot_romeo.jpg",
        miniPicture: "media/boards/dfrobot_romeo_mini.jpg",
        miniPicture_hor: "media/boards/dfrobot_romeo_mini_hor.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["0 (Rx) ; 1 (Tx)", "0"]],
        upload_arg: "arduino:avr:leonardo",
        help_link: "https://www.dfrobot.com/product-844.html"
    },
    dfrobot_romeo_ble: {
        description: "RoMeo BLE",
        cpu: "atmega328p",
        speed: "115200",
        digital: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13"],
        dropdownDigital: [["0", "0"], ["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"]],
        PWM: ["3", "5", "6", "9", "10", "11", "13"],
        dropdownPWM: [["3", "3"], ["5", "5"], ["6", "6"], ["9", "9"], ["10", "10"], ["11", "11"], ["13", "13"]],
        analog: ["A0", "A1", "A2", "A3", "A4", "A5", "A6", "A7"],
        dropdownAnalog: [["A0", "A0"], ["A1", "A1"], ["A2", "A2"], ["A3", "A3"], ["A4", "A4"], ["A5", "A5"], ["A6", "A6"], ["A7", "A7"]],
        /*irqonchange: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"],["A0", "14"], ["A1", "15"], ["A2", "16"], ["A3", "17"], ["A4", "18"], ["A5", "19"]],*/
        I2C: ["A4", "A5"],
        SPI: [["10 (SS)", "10"], ["11 (MOSI)", "11"], ["12 (MISO)", "12"], ["13 (SCK)", "13"]],
        interrupt: ["2", "3"],
        picture: "media/boards/dfrobot_romeo_ble.jpg",
        miniPicture: "media/boards/dfrobot_romeo_ble_mini.jpg",
        miniPicture_hor: "media/boards/dfrobot_romeo_ble_mini_hor.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["0 (Rx) ; 1 (Tx)", "0"]],
        upload_arg: "arduino:avr:uno",
        help_link: "https://www.dfrobot.com/product-1176.html"
    },
    esp8266: {
        description: "ESP8266",
        cpu: "esp8266",
        speed: "921600",
        digital: ["1", "2", "3", "4", "5", "12", "13", "14", "15", "16", "17", "18", "19", "21", "22", "23", "25", "26", "27", "32", "33", "34", "35", "36", "39"],
        dropdownDigital: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["12", "12"], ["13", "13"], ["14", "14"], ["15", "15"], ["16", "16"], ["17", "17"], ["18", "18"], ["19", "19"], ["21", "21"], ["22", "22"], ["23", "23"], ["25", "25"], ["26", "26"], ["27", "27"], ["32", "32"], ["33", "33"], ["34", "34"], ["35", "35"], ["36", "36"], ["39", "39"]],
        PWM: ["1", "2", "3", "4", "5", "12", "13", "14", "15", "16", "17", "18", "19", "21", "22", "23", "25", "26", "27", "32", "33", "34", "35", "36", "39"],
        dropdownPWM: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["12", "12"], ["13", "13"], ["14", "14"], ["15", "15"], ["16", "16"], ["17", "17"], ["18", "18"], ["19", "19"], ["21", "21"], ["22", "22"], ["23", "23"], ["25", "25"], ["26", "26"], ["27", "27"], ["32", "32"], ["33", "33"], ["34", "34"], ["35", "35"], ["36", "36"], ["39", "39"]],
        analog: ["A0"],
        dropdownAnalog: [["A0", "A0"]],
        I2C: ["21", "22"],
        SPI: [["13 (MOSI)", "13"], ["12 (MISO)", "12"], ["14 (SCK)", "14"]],
        interrupt: ["2", "3"],
        picture: "media/boards/esp8266.jpg",
        miniPicture: "media/boards/esp8266_mini.jpg",
        miniPicture_hor: "media/boards/esp8266_mini_hor.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["Rx0 ; Tx0", "3 ; 1"], ["Rx1 ; Tx1", "28 ; 29"], ["Rx2 ; Tx2", "16 ; 17"]],
        upload_arg: "esp8266:esp8266",
        help_link: "https://fr.wikipedia.org/wiki/ESP8266"
    },
    esp32: {
        description: "ESP32",
        cpu: "esp32",
        speed: "921600",
        digital: ["1", "2", "3", "4", "5", "12", "13", "14", "15", "16", "17", "18", "19", "21", "22", "23", "25", "26", "27", "32", "33", "34", "35", "36", "39"],
        dropdownDigital: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["12", "12"], ["13", "13"], ["14", "14"], ["15", "15"], ["16", "16"], ["17", "17"], ["18", "18"], ["19", "19"], ["21", "21"], ["22", "22"], ["23", "23"], ["25", "25"], ["26", "26"], ["27", "27"], ["32", "32"], ["33", "33"], ["34", "34"], ["35", "35"], ["36", "36"], ["39", "39"]],
        PWM: ["1", "2", "3", "4", "5", "12", "13", "14", "15", "16", "17", "18", "19", "21", "22", "23", "25", "26", "27", "32", "33", "34", "35", "36", "39"],
        dropdownPWM: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["12", "12"], ["13", "13"], ["14", "14"], ["15", "15"], ["16", "16"], ["17", "17"], ["18", "18"], ["19", "19"], ["21", "21"], ["22", "22"], ["23", "23"], ["25", "25"], ["26", "26"], ["27", "27"], ["32", "32"], ["33", "33"], ["34", "34"], ["35", "35"], ["36", "36"], ["39", "39"]],
        analog: ["36", "39", "32", "33", "34", "35"],
        dropdownAnalog: [["A0", "36"], ["A1", "39"], ["A2", "32"], ["A3", "33"], ["A4", "34"], ["A5", "35"]],
        I2C: ["21", "22"],
        SPI: [["15 (HSPI CS0)", "15"], ["13 (HSPI MOSI)", "13"], ["12 (HSPI MISO)", "12"], ["14 (HSPI CLK)", "14"], ["5 (VSPI CS0)", "5"], ["23 (VSPI MOSI)", "23"], ["19 (VSPI MISO)", "19"], ["18 (VSPI CLK)", "18"]],
        interrupt: ["1", "2", "3", "4", "5", "12", "13", "14", "15", "16", "17", "18", "19", "21", "22", "23", "25", "26", "27", "32", "33", "34", "35", "36", "39"],
        picture: "media/boards/esp32.jpg",
        miniPicture: "media/boards/esp32_mini.jpg",
        miniPicture_hor: "media/boards/esp32_mini_hor.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["Rx0 ; Tx0", "3 ; 1"], ["Rx1 ; Tx1", "28 ; 29"], ["Rx2 ; Tx2", "16 ; 17"]],
        upload_arg: "esp32:esp32:esp32",
        help_link: "https://en.wikipedia.org/wiki/ESP32"
    },
    peguino_uno_nano: {
        description: "Peguino Uno Nano",
        cpu: "atmega328p",
        speed: "115200",
        digital: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13"],
        dropdownDigital: [["0", "0"], ["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"]],
        PWM: ["3", "5", "6", "9", "10", "11"],
        dropdownPWM: [["3", "3"], ["5", "5"], ["6", "6"], ["9", "9"], ["10", "10"], ["11", "11"]],
        analog: ["A0", "A1", "A2", "A3", "A4", "A5", "A6", "A7"],
        dropdownAnalog: [["A0", "A0"], ["A1", "A1"], ["A2", "A2"], ["A3", "A3"], ["A4", "A4"], ["A5", "A5"], ["A6", "A6"], ["A7", "A7"]],
        /*irqonchange: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"],["A0", "14"], ["A1", "15"], ["A2", "16"], ["A3", "17"], ["A4", "18"], ["A5", "19"]],*/
        I2C: ["A4", "A5"],
        SPI: [["10 (SS)", "10"], ["11 (MOSI)", "11"], ["12 (MISO)", "12"], ["13 (SCK)", "13"]],
        interrupt: ["2", "3"],
        picture: "media/boards/peguino_uno_nano.jpg",
        miniPicture: "media/boards/peguino_uno_nano_mini.jpg",
        miniPicture_hor: "media/boards/peguino_uno_nano_mini_hor.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["0 (Rx) ; 1 (Tx)", "0"]],
        upload_arg: "arduino:avr:nano:cpu=atmega328old",
        help_link: "https://www.peguino.com/chat/thread-40.html"
    },
    peguino_uno_esp32: {
        description: "Peguino Uno ESP32",
        cpu: "esp32",
        speed: "921600",
        digital: ["1", "2", "3", "4", "5", "12", "13", "14", "15", "16", "17", "18", "19", "21", "22", "23", "25", "26", "27", "32", "33", "34", "35", "36", "39"],
        dropdownDigital: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["12", "12"], ["13", "13"], ["14", "14"], ["15", "15"], ["16", "16"], ["17", "17"], ["18", "18"], ["19", "19"], ["21", "21"], ["22", "22"], ["23", "23"], ["25", "25"], ["26", "26"], ["27", "27"], ["32", "32"], ["33", "33"], ["34", "34"], ["35", "35"], ["36", "36"], ["39", "39"]],
        PWM: ["1", "2", "3", "4", "5", "12", "13", "14", "15", "16", "17", "18", "19", "21", "22", "23", "25", "26", "27", "32", "33", "34", "35", "36", "39"],
        dropdownPWM: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["12", "12"], ["13", "13"], ["14", "14"], ["15", "15"], ["16", "16"], ["17", "17"], ["18", "18"], ["19", "19"], ["21", "21"], ["22", "22"], ["23", "23"], ["25", "25"], ["26", "26"], ["27", "27"], ["32", "32"], ["33", "33"], ["34", "34"], ["35", "35"], ["36", "36"], ["39", "39"]],
        analog: ["36", "39", "32", "33", "34", "35"],
        dropdownAnalog: [["A0", "36"], ["A1", "39"], ["A2", "32"], ["A3", "33"], ["A4", "34"], ["A5", "35"]],
        I2C: ["21", "22"],
        SPI: [["15 (HSPI CS0)", "15"], ["13 (HSPI MOSI)", "13"], ["12 (HSPI MISO)", "12"], ["14 (HSPI CLK)", "14"], ["5 (VSPI CS0)", "5"], ["23 (VSPI MOSI)", "23"], ["19 (VSPI MISO)", "19"], ["18 (VSPI CLK)", "18"]],
        interrupt: ["1", "2", "3", "4", "5", "12", "13", "14", "15", "16", "17", "18", "19", "21", "22", "23", "25", "26", "27", "32", "33", "34", "35", "36", "39"],
        picture: "media/boards/peguino_uno_esp32.jpg",
        miniPicture: "media/boards/peguino_uno_esp32_mini.jpg",
        miniPicture_hor: "media/boards/peguino_uno_esp32_mini_hor.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["Rx0 ; Tx0", "3 ; 1"], ["Rx1 ; Tx1", "28 ; 29"], ["Rx2 ; Tx2", "16 ; 17"]],
        upload_arg: "esp32:esp32:esp32",
        help_link: "https://www.peguino.com/chat/thread-39.html"
    },
    teensy2: {
        description: "Teensy 2.0",
        cpu: "atmega32u4",
        speed: "9600",
        digital: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15", "16", "17", "18", "19", "20", "21", "22", "23"],
        dropdownDigital: [["0", "0"], ["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"], ["14", "14"], ["15", "15"], ["16", "16"], ["7", "17"], ["18", "18"], ["19", "19"], ["20", "20"], ["21", "21"], ["22", "22"], ["23", "23"]],
        PWM: ["4", "5", "9", "10", "12", "14", "15"],
        dropdownPWM: [["4", "4"], ["5", "5"], ["9", "9"], ["10", "10"], ["12", "12"], ["14", "14"], ["15", "15"]],
        analog: ["11", "12", "13", "14", "15", "16", "17", "18", "19", "20", "21", "22"],
        dropdownAnalog: [["A0", "21"], ["A1", "20"], ["A2", "19"], ["A3", "18"], ["A4", "17"], ["A5", "16"], ["A6", "15"], ["A7", "14"], ["A8", "13"], ["A9", "12"], ["A10", "11"], ["A11", "22"]],
        I2C: [],
        SPI: [],
        interrupt: ["5", "6", "7", "8"],
        picture: "media/boards/teensy2_pinout.png",
        miniPicture: "media/boards/teensy2_pinout.png",
        miniPicture_hor: "media/boards/teensy2_pinout.png",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["Rx ; Tx", "7 ; 8"]],
        upload_arg: "arduino:avr:micro",
        help_link: "https://www.pjrc.com/teensy/pinout.html"
    },
    teensy2pp: {
        description: "Teensy++ 2.0",
        cpu: "AT90USB1286",
        speed: "9600",
        digital: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15", "16", "17", "18", "19", "20", "21", "22", "23", "24", "25", "26", "27", "38", "39", "40", "41", "42", "43", "44", "45"],
        dropdownDigital: [["0", "0"], ["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"], ["14", "14"], ["15", "15"], ["16", "16"], ["17", "17"], ["18", "18"], ["19", "19"], ["20", "20"], ["21", "21"], ["22", "22"], ["23", "23"], ["24", "24"], ["25", "25"], ["26", "26"], ["27", "27"], ["38", "38"], ["39", "39"], ["40", "40"], ["41", "41"], ["42", "42"], ["43", "43"], ["44", "44"], ["45", "45"]],
        PWM: ["0", "1", "14", "15", "16", "24", "25", "26", "27"],
        dropdownPWM: [["0", "0"], ["1", "1"], ["14", "14"], ["15", "15"], ["16", "16"], ["24", "24"], ["25", "25"], ["26", "26"], ["27", "27"]],
        analog: ["38", "39", "40", "41", "42", "43", "44", "45"],
        dropdownAnalog: [["A0", "38"], ["A1", "39"], ["A2", "40"], ["A3", "41"], ["A4", "42"], ["A5", "43"], ["A6", "44"], ["A7", "45"]],
        I2C: [],
        SPI: [],
        interrupt: ["0", "1", "2", "3", "18", "19"],
        picture: "media/boards/teensy2++_pinout.png",
        miniPicture: "media/boards/teensy2++_pinout.png",
        miniPicture_hor: "media/boards/teensy2++_pinout.png",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["Rx ; Tx", "2 ; 3"]],
        upload_arg: "",
        help_link: "https://www.pjrc.com/teensy/pinout.html"
    },
    kit_microbit: {
        description: "Micro:bit",
        cpu: "atmega328p",
        speed: "115200",
        digital: ["1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13"],
        dropdownDigital: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"]],
        PWM: ["3", "5", "6", "9", "10", "11"],
        dropdownPWM: [["3", "3"], ["5", "5"], ["6", "6"], ["9", "9"], ["10", "10"], ["11", "11"]],
        analog: ["A0", "A1", "A2", "A3", "A4", "A5", "A6", "A7"],
        dropdownAnalog: [["A0", "A0"], ["A1", "A1"], ["A2", "A2"], ["A3", "A3"], ["A4", "A4"], ["A5", "A5"]],
        /*irqonchange: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"],["A0", "14"], ["A1", "15"], ["A2", "16"], ["A3", "17"], ["A4", "18"], ["A5", "19"]],*/
        I2C: ["A4", "A5"],
        SPI: [["10 (SS)", "10"], ["11 (MOSI)", "11"], ["12 (MISO)", "12"], ["13 (SCK)", "13"]],
        interrupt: ["2", "3"],
        picture: "media/boards/kit_microbit.jpg",
        miniPicture: "media/boards/kit_microbit_mini.jpg",
        miniPicture_hor: "media/boards/kit_microbit_mini_hor.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["0 (Rx) ; 1 (Tx)", "0"]],
        upload_arg: "arduino:avr:uno",
        help_link: "https://microbit.org/fr/guide/features/"
    },
    kit_microfeux: {
        description: "Micro-feux Jeulin",
        cpu: "atmega328p",
        speed: "115200",
        digital: ["1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13"],
        dropdownDigital: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"]],
        PWM: ["3", "5", "6", "9", "10", "11"],
        dropdownPWM: [["3", "3"], ["5", "5"], ["6", "6"], ["9", "9"], ["10", "10"], ["11", "11"]],
        analog: ["A0", "A1", "A2", "A3", "A4", "A5", "A6", "A7"],
        dropdownAnalog: [["A0", "A0"], ["A1", "A1"], ["A2", "A2"], ["A3", "A3"], ["A4", "A4"], ["A5", "A5"]],
        /*irqonchange: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"],["A0", "14"], ["A1", "15"], ["A2", "16"], ["A3", "17"], ["A4", "18"], ["A5", "19"]],*/
        I2C: ["A4", "A5"],
        SPI: [["10 (SS)", "10"], ["11 (MOSI)", "11"], ["12 (MISO)", "12"], ["13 (SCK)", "13"]],
        interrupt: ["2", "3"],
        picture: "media/boards/kit_microfeux.jpg",
        miniPicture: "media/boards/kit_microfeux_mini.jpg",
        miniPicture_hor: "media/boards/kit_microfeux_mini_hor.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["0 (Rx) ; 1 (Tx)", "0"]],
        upload_arg: "arduino:avr:uno",
        help_link: "https://www.qwant.com/?q=microfeux%20jeulin&t=all"
    },
    //20191010
    kit_microsaurus: {
        description: "MicroSaurus",
        cpu: "atmega328p",
        speed: "115200",
        digital: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13"],
        dropdownDigital: [["0", "0"], ["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"]],
        PWM: ["3", "5", "6", "9", "10", "11"],
        dropdownPWM: [["3", "3"], ["5", "5"], ["6", "6"], ["9", "9"], ["10", "10"], ["11", "11"]],
        analog: ["A0", "A1", "A2", "A3", "A4", "A5"],
        dropdownAnalog: [["A0", "A0"], ["A1", "A1"], ["A2", "A2"], ["A3", "A3"], ["A4", "A4"], ["A5", "A5"]],
        /*irqonchange: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"],["A0", "14"], ["A1", "15"], ["A2", "16"], ["A3", "17"], ["A4", "18"], ["A5", "19"]],*/
        I2C: ["A4", "A5"],
        SPI: [["10 (SS)", "10"], ["11 (MOSI)", "11"], ["12 (MISO)", "12"], ["13 (SCK)", "13"]],
        interrupt: ["2", "3"],
        picture: "media/boards/kit_microsaurus.jpg",
        miniPicture: "media/boards/kit_microsaurus_mini.jpg",
        miniPicture_hor: "media/boards/kit_microsaurus_mini_hor.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["Rx ; Tx", "0"]],
        upload_arg: "arduino:avr:uno",
        help_link: " https://meuse.co.jp/eshop/cog/cog_tutorial_index/"
    },
    kit_micromachine: {
        description: "MicroMachine",
        cpu: "esp32",
        speed: "115200",
        digital: ["1", "2", "3", "4", "5", "12", "13", "14", "15", "16", "17", "18", "19", "21", "22", "23", "25", "26", "27", "32", "33", "34", "35", "36", "39"],
        dropdownDigital: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["12", "12"], ["13", "13"], ["14", "14"], ["15", "15"], ["16", "16"], ["17", "17"], ["18", "18"], ["19", "19"], ["21", "21"], ["22", "22"], ["23", "23"], ["25", "25"], ["26", "26"], ["27", "27"], ["32", "32"], ["33", "33"], ["34", "34"], ["35", "35"], ["36", "36"], ["39", "39"]],
        PWM: ["1", "2", "3", "4", "5", "12", "13", "14", "15", "16", "17", "18", "19", "21", "22", "23", "25", "26", "27", "32", "33", "34", "35", "36", "39"],
        dropdownPWM: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["12", "12"], ["13", "13"], ["14", "14"], ["15", "15"], ["16", "16"], ["17", "17"], ["18", "18"], ["19", "19"], ["21", "21"], ["22", "22"], ["23", "23"], ["25", "25"], ["26", "26"], ["27", "27"], ["32", "32"], ["33", "33"], ["34", "34"], ["35", "35"], ["36", "36"], ["39", "39"]],
        analog: ["36", "39", "32", "33", "34", "35"],
        dropdownAnalog: [["A0", "36"], ["A1", "39"], ["A2", "32"], ["A3", "33"], ["A4", "34"], ["A5", "35"]],
        I2C: ["21", "22"],
        SPI: [["15 (HSPI CS0)", "15"], ["13 (HSPI MOSI)", "13"], ["12 (HSPI MISO)", "12"], ["14 (HSPI CLK)", "14"], ["5 (VSPI CS0)", "5"], ["23 (VSPI MOSI)", "23"], ["19 (VSPI MISO)", "19"], ["18 (VSPI CLK)", "18"]],
        interrupt: ["1", "2", "3", "4", "5", "12", "13", "14", "15", "16", "17", "18", "19", "21", "22", "23", "25", "26", "27", "32", "33", "34", "35", "36", "39"],
        picture: "media/boards/kit_micromachine.jpg",
        miniPicture: "media/boards/kit_micromachine_mini.jpg",
        miniPicture_hor: "media/boards/kit_micromachine_mini_hor.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["Rx0 ; Tx0", "3 ; 1"], ["Rx1 ; Tx1", "28 ; 29"], ["Rx2 ; Tx2", "16 ; 17"]],
        upload_arg: "esp32:esp32:esp32",
        help_link: "https://meuse.co.jp/eshop/cog/cog_tutorial_index/"
    },
    //20191010,
    kit_otto_diy: {
        description: "Otto DIY",
        cpu: "atmega328p",
        speed: "115200",
        digital: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13"],
        dropdownDigital: [["0", "0"], ["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"]],
        PWM: ["3", "5", "6", "9", "10", "11"],
        dropdownPWM: [["3", "3"], ["5", "5"], ["6", "6"], ["9", "9"], ["10", "10"], ["11", "11"]],
        analog: ["A0", "A1", "A2", "A3", "A4", "A5", "A6", "A7"],
        dropdownAnalog: [["A0", "A0"], ["A1", "A1"], ["A2", "A2"], ["A3", "A3"], ["A4", "A4"], ["A5", "A5"], ["A6", "A6"], ["A7", "A7"]],
        /*irqonchange: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"],["A0", "14"], ["A1", "15"], ["A2", "16"], ["A3", "17"], ["A4", "18"], ["A5", "19"]],*/
        I2C: ["A4", "A5"],
        SPI: [["10 (SS)", "10"], ["11 (MOSI)", "11"], ["12 (MISO)", "12"], ["13 (SCK)", "13"]],
        interrupt: ["2", "3"],
        picture: "media/boards/kit_otto_diy.jpg",
        miniPicture: "media/boards/kit_otto_diy_mini.jpg",
        miniPicture_hor: "media/boards/kit_otto_diy_mini_hor.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["0 (Rx) ; 1 (Tx)", "0"]],
        upload_arg: "arduino:avr:nano:cpu=atmega328",
        help_link: "https://wikifactory.com/+OttoDIY/otto-diy/file/Instruction%20manual/OttoDIY_Manual_V9.pdf"
    },
    kit_peguino_bot1: {
        description: "Peguino Bot 1",
        cpu: "esp32",
        speed: "921600",
        digital: ["1", "2", "3", "4", "5", "12", "13", "14", "15", "16", "17", "18", "19", "21", "22", "23", "25", "26", "27", "32", "33", "34", "35", "36", "39"],
        dropdownDigital: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["12", "12"], ["13", "13"], ["14", "14"], ["15", "15"], ["16", "16"], ["17", "17"], ["18", "18"], ["19", "19"], ["21", "21"], ["22", "22"], ["23", "23"], ["25", "25"], ["26", "26"], ["27", "27"], ["32", "32"], ["33", "33"], ["34", "34"], ["35", "35"], ["36", "36"], ["39", "39"]],
        PWM: ["1", "2", "3", "4", "5", "12", "13", "14", "15", "16", "17", "18", "19", "21", "22", "23", "25", "26", "27", "32", "33", "34", "35", "36", "39"],
        dropdownPWM: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["12", "12"], ["13", "13"], ["14", "14"], ["15", "15"], ["16", "16"], ["17", "17"], ["18", "18"], ["19", "19"], ["21", "21"], ["22", "22"], ["23", "23"], ["25", "25"], ["26", "26"], ["27", "27"], ["32", "32"], ["33", "33"], ["34", "34"], ["35", "35"], ["36", "36"], ["39", "39"]],
        analog: ["36", "39", "32", "33", "34", "35"],
        dropdownAnalog: [["A0", "36"], ["A1", "39"], ["A2", "32"], ["A3", "33"], ["A4", "34"], ["A5", "35"]],
        I2C: ["21", "22"],
        SPI: [["15 (HSPI CS0)", "15"], ["13 (HSPI MOSI)", "13"], ["12 (HSPI MISO)", "12"], ["14 (HSPI CLK)", "14"], ["5 (VSPI CS0)", "5"], ["23 (VSPI MOSI)", "23"], ["19 (VSPI MISO)", "19"], ["18 (VSPI CLK)", "18"]],
        interrupt: ["1", "2", "3", "4", "5", "12", "13", "14", "15", "16", "17", "18", "19", "21", "22", "23", "25", "26", "27", "32", "33", "34", "35", "36", "39"],
        picture: "media/boards/kit_peguino_bot1.jpg",
        miniPicture: "media/boards/kit_peguino_bot1_mini.jpg",
        miniPicture_hor: "media/boards/kit_peguino_bot1_mini_hor.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["Rx0 ; Tx0", "3 ; 1"], ["Rx1 ; Tx1", "28 ; 29"], ["Rx2 ; Tx2", "16 ; 17"]],
        upload_arg: "esp32:esp32:esp32",
        help_link: "https://www.peguino.com/chat/thread-65.html"
    },
    kit_petitbot: {
        description: "Petit Bot",
        cpu: "",
        speed: "",
        digital: ["D0", "D1", "D2", "D3", "D4", "D5", "D6", "D7", "D8"],
        dropdownDigital: [["D0", "D0"], ["D1", "D1"], ["D2", "D2"], ["D3", "D3"], ["D4", "D4"], ["D5", "D5"], ["D6", "D6"], ["D7", "D7"], ["D8", "D8"]],
        PWM: ["D0", "D1", "D2", "D3", "D4", "D5", "D6", "D7", "D8"],
        dropdownPWM: [["D0", "D0"], ["D1", "D1"], ["D2", "D2"], ["D3", "D3"], ["D4", "D4"], ["D5", "D5"], ["D6", "D6"], ["D7", "D7"], ["D8", "D8"]],
        analog: ["A0"],
        dropdownAnalog: [["A0", "A0"]],
        I2C: ["D1", "D2", "D3", "D4", "D5", "D6", "D7", "D8"],
        SPI: ["D1", "D2", "D3", "D4", "D5", "D6", "D7", "D8"],
        interrupt: ["D2", "D3"],
        picture: "media/boards/kit_petitbot.jpg",
        miniPicture: "media/boards/kit_petitbot_mini.jpg",
        miniPicture_hor: "media/boards/kit_petitbot_mini_hor.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["Rx ; Tx", "0"]],
        upload_arg: "B:\LiberkeyLivet\MyApps\arduino\portable\packages\esp8266\tools\esptool\0.4.8/esptool.exe -vv -cd nodemcu -cb 921600 -cp COM3 -ca 0x00000 -cf B:\LiberkeyLivet\MyApps\arduino\build/petitbot_v3.ino.bin ",
        help_link: "https://github.com/julienrat/petitbot/blob/master/manuel_tech_petitbot.pdf"
    },
    kit_robobox_1_1: {
        description: "mini-alarme v1",
        cpu: "atmega328p",
        speed: "115200",
        digital: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13"],
        dropdownDigital: [["0", "0"], ["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"]],
        PWM: ["3", "5", "6", "9", "10", "11"],
        dropdownPWM: [["3", "3"], ["5", "5"], ["6", "6"], ["9", "9"], ["10", "10"], ["11", "11"]],
        analog: ["A0", "A1", "A2", "A3", "A4", "A5"],
        dropdownAnalog: [["A0", "A0"], ["A1", "A1"], ["A2", "A2"], ["A3", "A3"], ["A4", "A4"], ["A5", "A5"]],
        /*irqonchange: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"],["A0", "14"], ["A1", "15"], ["A2", "16"], ["A3", "17"], ["A4", "18"], ["A5", "19"]],*/
        I2C: ["A4", "A5"],
        SPI: [["10 (SS)", "10"], ["11 (MOSI)", "11"], ["12 (MISO)", "12"], ["13 (SCK)", "13"]],
        interrupt: ["2", "3"],
        picture: "media/boards/robobox_box1.1.jpg",
        miniPicture: "media/boards/robobox_box1.1.jpg",
        miniPicture_hor: "media/boards/robobox_box1.1.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["0 (Rx) ; 1 (Tx)", "0"]],
        upload_arg: "arduino:avr:uno",
        help_link: "https://www.arduino.cc/en/Main/ArduinoBoardUno"
    },
    kit_robobox_2_1: {
        description: "bras articulé v1",
        cpu: "atmega328p",
        speed: "115200",
        digital: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13"],
        dropdownDigital: [["0", "0"], ["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"]],
        PWM: ["3", "5", "6", "9", "10", "11"],
        dropdownPWM: [["3", "3"], ["5", "5"], ["6", "6"], ["9", "9"], ["10", "10"], ["11", "11"]],
        analog: ["A0", "A1", "A2", "A3", "A4", "A5"],
        dropdownAnalog: [["A0", "A0"], ["A1", "A1"], ["A2", "A2"], ["A3", "A3"], ["A4", "A4"], ["A5", "A5"]],
        /*irqonchange: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"],["A0", "14"], ["A1", "15"], ["A2", "16"], ["A3", "17"], ["A4", "18"], ["A5", "19"]],*/
        I2C: ["A4", "A5"],
        SPI: [["10 (SS)", "10"], ["11 (MOSI)", "11"], ["12 (MISO)", "12"], ["13 (SCK)", "13"]],
        interrupt: ["2", "3"],
        picture: "media/boards/robobox_box2.1.jpg",
        miniPicture: "media/boards/robobox_box2.1.jpg",
        miniPicture_hor: "media/boards/robobox_box2.1.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["0 (Rx) ; 1 (Tx)", "0"]],
        upload_arg: "arduino:avr:uno",
        help_link: "https://www.arduino.cc/en/Main/ArduinoBoardUno"
    },
    kit_robobox_3_1: {
        description: "robot chien v1",
        cpu: "atmega328p",
        speed: "115200",
        digital: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13"],
        dropdownDigital: [["0", "0"], ["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"]],
        PWM: ["3", "5", "6", "9", "10", "11"],
        dropdownPWM: [["3", "3"], ["5", "5"], ["6", "6"], ["9", "9"], ["10", "10"], ["11", "11"]],
        analog: ["A0", "A1", "A2", "A3", "A4", "A5"],
        dropdownAnalog: [["A0", "A0"], ["A1", "A1"], ["A2", "A2"], ["A3", "A3"], ["A4", "A4"], ["A5", "A5"]],
        /*irqonchange: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"],["A0", "14"], ["A1", "15"], ["A2", "16"], ["A3", "17"], ["A4", "18"], ["A5", "19"]],*/
        I2C: ["A4", "A5"],
        SPI: [["10 (SS)", "10"], ["11 (MOSI)", "11"], ["12 (MISO)", "12"], ["13 (SCK)", "13"]],
        interrupt: ["2", "3"],
        picture: "media/boards/robobox_box3.1.jpg",
        miniPicture: "media/boards/robobox_box3.1.jpg",
        miniPicture_hor: "media/boards/robobox_box3.1.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["0 (Rx) ; 1 (Tx)", "0"]],
        upload_arg: "arduino:avr:uno",
        help_link: "https://www.arduino.cc/en/Main/ArduinoBoardUno"
    },
    kit_robobox_4_1: {
        description: "robot voiture v1",
        cpu: "atmega328p",
        speed: "115200",
        digital: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13"],
        dropdownDigital: [["0", "0"], ["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"]],
        PWM: ["3", "5", "6", "9", "10", "11"],
        dropdownPWM: [["3", "3"], ["5", "5"], ["6", "6"], ["9", "9"], ["10", "10"], ["11", "11"]],
        analog: ["A0", "A1", "A2", "A3", "A4", "A5"],
        dropdownAnalog: [["A0", "A0"], ["A1", "A1"], ["A2", "A2"], ["A3", "A3"], ["A4", "A4"], ["A5", "A5"]],
        /*irqonchange: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"],["A0", "14"], ["A1", "15"], ["A2", "16"], ["A3", "17"], ["A4", "18"], ["A5", "19"]],*/
        I2C: ["A4", "A5"],
        SPI: [["10 (SS)", "10"], ["11 (MOSI)", "11"], ["12 (MISO)", "12"], ["13 (SCK)", "13"]],
        interrupt: ["2", "3"],
        picture: "media/boards/robobox_box4.1.jpg",
        miniPicture: "media/boards/robobox_box4.1.jpg",
        miniPicture_hor: "media/boards/robobox_box4.1.jpg",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["0 (Rx) ; 1 (Tx)", "0"]],
        upload_arg: "arduino:avr:uno",
        help_link: "https://www.arduino.cc/en/Main/ArduinoBoardUno"
    },
    kit_robobox_1_2: {
        description: "mini alarme v2",
        cpu: "atmega328p",
        speed: "115200",
        digital: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13"],
        dropdownDigital: [["0", "0"], ["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"]],
        PWM: ["3", "5", "6", "9", "10", "11"],
        dropdownPWM: [["3", "3"], ["5", "5"], ["6", "6"], ["9", "9"], ["10", "10"], ["11", "11"]],
        analog: ["A0", "A1", "A2", "A3", "A4", "A5", "A6", "A7"],
        dropdownAnalog: [["A0", "A0"], ["A1", "A1"], ["A2", "A2"], ["A3", "A3"], ["A4", "A4"], ["A5", "A5"], ["A6", "A6"], ["A7", "A7"]],
        /*irqonchange: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"],["A0", "14"], ["A1", "15"], ["A2", "16"], ["A3", "17"], ["A4", "18"], ["A5", "19"]],*/
        I2C: ["A4", "A5"],
        SPI: [["10 (SS)", "10"], ["11 (MOSI)", "11"], ["12 (MISO)", "12"], ["13 (SCK)", "13"]],
        interrupt: ["2", "3"],
        picture: "media/boards/robobox_box1.2.png",
        miniPicture: "media/boards/robobox_box1.2.png",
        miniPicture_hor: "media/boards/robobox_box1.2.png",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["0 (Rx) ; 1 (Tx)", "0"]],
        upload_arg: "arduino:avr:nano:cpu=atmega328",
        help_link: "https://www.arduino.cc/en/Main/ArduinoBoardNano"
    },
    kit_robobox_2_2: {
        description: "bras articulé v2",
        cpu: "atmega328p",
        speed: "115200",
        digital: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13"],
        dropdownDigital: [["0", "0"], ["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"]],
        PWM: ["3", "5", "6", "9", "10", "11"],
        dropdownPWM: [["3", "3"], ["5", "5"], ["6", "6"], ["9", "9"], ["10", "10"], ["11", "11"]],
        analog: ["A0", "A1", "A2", "A3", "A4", "A5", "A6", "A7"],
        dropdownAnalog: [["A0", "A0"], ["A1", "A1"], ["A2", "A2"], ["A3", "A3"], ["A4", "A4"], ["A5", "A5"], ["A6", "A6"], ["A7", "A7"]],
        /*irqonchange: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"],["A0", "14"], ["A1", "15"], ["A2", "16"], ["A3", "17"], ["A4", "18"], ["A5", "19"]],*/
        I2C: ["A4", "A5"],
        SPI: [["10 (SS)", "10"], ["11 (MOSI)", "11"], ["12 (MISO)", "12"], ["13 (SCK)", "13"]],
        interrupt: ["2", "3"],
        picture: "media/boards/robobox_box2.2.png",
        miniPicture: "media/boards/robobox_box2.2.png",
        miniPicture_hor: "media/boards/robobox_box2.2.png",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["0 (Rx) ; 1 (Tx)", "0"]],
        upload_arg: "arduino:avr:nano:cpu=atmega328",
        help_link: "https://www.arduino.cc/en/Main/ArduinoBoardNano"
    },
    kit_robobox_3_2: {
        description: "robot chien v2",
        cpu: "atmega328p",
        speed: "115200",
        digital: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13"],
        dropdownDigital: [["0", "0"], ["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"]],
        PWM: ["3", "5", "6", "9", "10", "11"],
        dropdownPWM: [["3", "3"], ["5", "5"], ["6", "6"], ["9", "9"], ["10", "10"], ["11", "11"]],
        analog: ["A0", "A1", "A2", "A3", "A4", "A5", "A6", "A7"],
        dropdownAnalog: [["A0", "A0"], ["A1", "A1"], ["A2", "A2"], ["A3", "A3"], ["A4", "A4"], ["A5", "A5"], ["A6", "A6"], ["A7", "A7"]],
        /*irqonchange: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"],["A0", "14"], ["A1", "15"], ["A2", "16"], ["A3", "17"], ["A4", "18"], ["A5", "19"]],*/
        I2C: ["A4", "A5"],
        SPI: [["10 (SS)", "10"], ["11 (MOSI)", "11"], ["12 (MISO)", "12"], ["13 (SCK)", "13"]],
        interrupt: ["2", "3"],
        picture: "media/boards/robobox_box3.2.png",
        miniPicture: "media/boards/robobox_box3.2.png",
        miniPicture_hor: "media/boards/robobox_box3.2.png",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["0 (Rx) ; 1 (Tx)", "0"]],
        upload_arg: "arduino:avr:nano:cpu=atmega328",
        help_link: "https://www.arduino.cc/en/Main/ArduinoBoardNano"
    },
    kit_robobox_4_2: {
        description: "voiture v2",
        cpu: "atmega328p",
        speed: "115200",
        digital: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13"],
        dropdownDigital: [["0", "0"], ["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"]],
        PWM: ["3", "5", "6", "9", "10", "11"],
        dropdownPWM: [["3", "3"], ["5", "5"], ["6", "6"], ["9", "9"], ["10", "10"], ["11", "11"]],
        analog: ["A0", "A1", "A2", "A3", "A4", "A5", "A6", "A7"],
        dropdownAnalog: [["A0", "A0"], ["A1", "A1"], ["A2", "A2"], ["A3", "A3"], ["A4", "A4"], ["A5", "A5"], ["A6", "A6"], ["A7", "A7"]],
        /*irqonchange: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"],["A0", "14"], ["A1", "15"], ["A2", "16"], ["A3", "17"], ["A4", "18"], ["A5", "19"]],*/
        I2C: ["A4", "A5"],
        SPI: [["10 (SS)", "10"], ["11 (MOSI)", "11"], ["12 (MISO)", "12"], ["13 (SCK)", "13"]],
        interrupt: ["2", "3"],
        picture: "media/boards/robobox_box4.2.png",
        miniPicture: "media/boards/robobox_box4.2.png",
        miniPicture_hor: "media/boards/robobox_box4.2.png",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["0 (Rx) ; 1 (Tx)", "0"]],
        upload_arg: "arduino:avr:nano:cpu=atmega328",
        help_link: "https://www.arduino.cc/en/Main/ArduinoBoardNano"
    },
    kit_robobox_5_2: {
        description: "voiture v2 et ",
        cpu: "atmega328p",
        speed: "115200",
        digital: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13"],
        dropdownDigital: [["0", "0"], ["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"]],
        PWM: ["3", "5", "6", "9", "10", "11"],
        dropdownPWM: [["3", "3"], ["5", "5"], ["6", "6"], ["9", "9"], ["10", "10"], ["11", "11"]],
        analog: ["A0", "A1", "A2", "A3", "A4", "A5", "A6", "A7"],
        dropdownAnalog: [["A0", "A0"], ["A1", "A1"], ["A2", "A2"], ["A3", "A3"], ["A4", "A4"], ["A5", "A5"], ["A6", "A6"], ["A7", "A7"]],
        /*irqonchange: [["1", "1"], ["2", "2"], ["3", "3"], ["4", "4"], ["5", "5"], ["6", "6"], ["7", "7"], ["8", "8"], ["9", "9"], ["10", "10"], ["11", "11"], ["12", "12"], ["13", "13"],["A0", "14"], ["A1", "15"], ["A2", "16"], ["A3", "17"], ["A4", "18"], ["A5", "19"]],*/
        I2C: ["A4", "A5"],
        SPI: [["10 (SS)", "10"], ["11 (MOSI)", "11"], ["12 (MISO)", "12"], ["13 (SCK)", "13"]],
        interrupt: ["2", "3"],
        picture: "media/boards/robobox_box5.2.png",
        miniPicture: "media/boards/robobox_box5.2.png",
        miniPicture_hor: "media/boards/robobox_box5.2.png",
        serial: [['300', '300'], ['600', '600'], ['1200', '1200'],
            ['2400', '2400'], ['4800', '4800'], ['9600', '9600'],
            ['14400', '14400'], ['19200', '19200'], ['28800', '28800'],
            ['31250', '31250'], ['38400', '38400'], ['57600', '57600'],
            ['115200', '115200']],
        serialPin: [["0 (Rx) ; 1 (Tx)", "0"]],
        upload_arg: "arduino:avr:nano:cpu=atmega328",
        help_link: "https://www.arduino.cc/en/Main/ArduinoBoardNano"
    }
};

//set default profile to arduino standard-compatible board
profile.defaultBoard = profile["none"];

/**
 * Ensure that PIN number exists in Digital array.
 * @param {string} text The user's text.
 * @return {?string} A string representing a valid PIN number, or null if invalid.
 */
Blockly.Arduino.pinDigitalValidator = function (text) {
    var pos = profile.defaultBoard.digital.indexOf(text);
    return (pos < 0) ? null : text;
};

Blockly.Arduino.pinInterruptValidator = function (text) {
    var pos = profile.defaultBoard.interrupt.indexOf(text);
    return (pos < 0) ? null : text;
};

Blockly.Arduino.pinSoftSerialValidator = function (text) {
    var pos = profile.defaultBoard.serialPin.indexOf(text);
    return (pos < 0) ? null : text;
};

/**
 * Ensure that PIN and next PIN numbers exists in Digital array.
 * @param {string} text The user's text.
 * @return {?string} A string representing a valid PIN number, or null if invalid.
 */
Blockly.Arduino.pinGroveDigitalValidator = function (text) {
    var pos = profile.defaultBoard.digital.indexOf(text);
    if (pos >= 0) {
        var NextPIN = parseInt(text) + 1;
        // check if NextPIN in bound
        pos = profile.defaultBoard.digital.indexOf(String(NextPIN));
    }
    return (pos < 0) ? null : text;
};

/**
 * Ensure that PIN number exists in PWM array.
 * @param {string} text The user's text.
 * @return {?string} A string representing a valid PIN number, or null if invalid.
 */
Blockly.Arduino.pinPWMValidator = function (text) {
    var pos = profile.defaultBoard.PWM.indexOf(text);
    return (pos < 0) ? null : text;
};

/**
 * Ensure that PIN number exists in Analog array.
 * @param {string} text The user's text.
 * @return {?string} A string representing a valid PIN number, or null if invalid.
 */
Blockly.Arduino.pinAnalogValidator = function (text) {
    var pos = profile.defaultBoard.analog.indexOf(text);
    return (pos < 0) ? null : text;
};

/**
 * Ensure that PIN and next PIN numbers exists in Digital array.
 * @param {string} text The user's text.
 * @return {?string} A string representing a valid PIN number, or null if invalid.
 */
Blockly.Arduino.pinGroveAnalogValidator = function (text) {
    var pos = profile.defaultBoard.analog.indexOf(text);
    if (pos >= 0) {
        var NextPIN = 'A' + (parseInt(text.slice(1, text.length)) + 1);
        // check if NextPIN in bound
        pos = profile.defaultBoard.analog.indexOf(String(NextPIN));
    }
    return (pos < 0) ? null : text;
};

/**
 * Ensure that PIN and next PIN numbers exists in Digital or Analog array.
 * @param {string} text The user's text.
 * @return {?string} A string representing a valid PIN number, or null if invalid.
 */
Blockly.Arduino.pinDualValidator = function (text) {
    var posa = profile.defaultBoard.analog.indexOf(text);
    var posd = profile.defaultBoard.digital.indexOf(text);
    var pos = posa + posd;
    return (pos < 0) ? null : text;
};